%==========================================================================
function [UDPts] = UDSampleRSVM(id, C_Range, G_Range, Stage, Pattern, Params, Center)
%--------------------------------------------------------------------------
% use UD table to determine the parameter values for model selection
% Inputs:
% C_Range   [1 x 2]  : [min_C, max_C]
% G_Range   [1 x 2]  : [min_gamma, max_gamma]  or e_Range   [1 x 2]  : [min_e, max_e] if id==1
% In Regression, the searching range of episilon is equal to [0 , 0.5]
% Stage     [1 x ?]  : the stage of nested UDs
% Pattern   [1 x 1]  : the UD pattern, 5-5runs,9-9runs,and 13-13runs
% Center    [1 x 2]  : the UD center for current stage 
%                      (default= center of the searching box)

% id        id=1: WSVM and SVM,     id=2: WRSVM
% Outputs:
% UDPts     [? x 2 (or 3)]  : the UD sampling points for current stage
% by K.Y. Lee
%--------------------------------------------------------------------------
   if(nargin < 7)
        Center = [sum(C_Range)/2,sum(G_Range)/2,0.25];
   end
    
    CLower=C_Range(1); CUpper=C_Range(2); CLen=CUpper-CLower; % lower bound, upper bound and length for C range
    GLower=G_Range(1); GUpper=G_Range(2); GLen=GUpper-GLower; % lower bound, upper bound and length for gamma range
    
    
    Center = Center(1:2);
    % 2-D Uniform Design table from http://www.math.hkbu.edu.hk/UniformDesign/
    UDTable_30 = [24, 25; 23, 6; 1, 12; 9, 18; 5, 16; 11, 7; 19, 4; 6, 9; 21, 11; 3, 3; 12, 14; 15, 20; 20, 30; 18, 15; 17, 24; 26, 2; 7, 29; 4, 21; 28, 13; 27, 28; 25, 17; 13, 27; 14, 1; 29, 8; 22, 19; 8, 5; 2, 26; 16, 10; 30, 22; 10, 23];
    UDTable_20 = [16, 15; 18, 19; 12, 1; 19, 3; 1, 9; 10, 7; 9, 20; 4, 13; 2, 18; 14, 10; 6, 16; 15, 5; 5, 6; 20, 12; 11, 14; 13, 17; 8, 4; 7, 11; 3, 2; 17 8 ];
    UDTable_13 = [7, 7; 5, 4; 12, 3; 2, 11; 9, 10; 6, 13; 3, 2; 11, 12; 13, 8; 10, 5; 1, 6; 4, 9; 8, 1];
    UDTable_10 = [2, 9; 8, 10; 10, 6; 4, 7; 1, 3; 5, 1; 9, 2; 7, 4; 6, 8; 3, 5];
    UDTable_9 = [5, 5; 1, 4; 7, 8; 2, 7; 3, 2; 9, 6; 8, 3; 6, 1; 4, 9];
    UDTable_5 = [3, 3; 1, 2; 2, 5; 4, 1; 5, 4];
    Table = []; eval(['Table = UDTable_', num2str(Pattern), ';']); % decide which UDtable to use
    
    UDPts =(Table-1)/(Pattern-1)/2^(Stage-1).*(ones(Pattern,1)*[CLen,GLen])+ones(Pattern,1)*[Center(1)-CLen/2^Stage Center(2)-GLen/2^Stage];   
    
    
    UDPts(find(UDPts(:,1)<CLower),1) = CLower;
    UDPts(find(UDPts(:,1)>CUpper),1) = CUpper;
    UDPts(find(UDPts(:,2)<GLower),2) = GLower;
    UDPts(find(UDPts(:,2)>GUpper),2) = GUpper;
    
    
end

